searchData={"items":[{"type":"module","title":"Shapex","doc":"Shapex is a tool to help you validate your maps.\n\nYou'll need to define a schema for a map and then you can use Shapex to validate it.","ref":"Shapex.html"},{"type":"module","title":"Built-in types - Shapex","doc":"There are some built-in types that you can use to define your schema:\n  - any\n  - atom\n  - boolean\n  - dict\n  - enum\n  - float\n  - integer\n  - list\n  - map\n  - nil\n  - number\n  - string","ref":"Shapex.html#module-built-in-types"},{"type":"module","title":"Schema DSL - Shapex","doc":"This DSL is used to simplify the creation of the contract that will be used for data validation.","ref":"Shapex.html#module-schema-dsl"},{"type":"module","title":"Key differences - Shapex","doc":"The differences between schema DSL and default function composition style are:\n\n```\n1 -> integer(eq: 1)\n1.0 -> float(eq: 1.0)\ntrue -> boolean(true)\n:atom -> atom(eq: :atom)\n\"string\" -> string(eq: \"string\")\n1 | 2 | 3 -> enum([integer(eq: 1), integer(eq: 2), integer(eq: 3)])\n%{name: \"John Doe\"} -> map(key: string(eq: \"John Doe\"))\n```\n\nThe schema DSL is more readable and easier to understand. This is difference for the big schema:\n\n#### Function composition\n```\nalias Shapex.Types, as: S\n\nanimal_schema = S.enum([\n  S.map(%{\n    name: S.string(),\n    family: S.atom(eq: :dog),\n    breed: S.enum([S.string(eq: \"Akita\"), S.string(eq: \"Husky\"), S.string(eq: \"Poodle\")])\n  }),\n  S.map(%{\n    name: S.string(),\n    family: S.atom(eq: :cat),\n    breed: S.enum([S.string(eq: \"Siamese\"), S.string(eq: \"Persian\"), S.string(eq: \"Maine Coon\")])\n  }),\n  S.map(%{\n    name: S.string(),\n    family: S.atom(eq: :owl),\n    genus: S.enum([S.string(eq: \"Athene\"), S.string(eq: \"Bubo\"), S.string(eq: \"Strix\")])\n  })\n])\n\n```\n\n#### Schema DSL\n\n```\n\nrequire Shapex\n\nanimal_schema = Shapex.schema(\n  %{\n    name: string(min_length: 2),\n    family: :dog,\n    breed: \"Akita\" | \"Husky\" | \"Poodle\"\n  }\n  | %{\n    name: string(min_length: 2),\n    family: :cat,\n    breed: \"Siamese\" | \"Persian\" | \"Maine Coon\"\n  }\n  | %{\n    name: string(min_length: 2),\n    family: :owl,\n    genus: \"Athene\" | \"Bubo\" | \"Strix\"\n  }\n)\n```","ref":"Shapex.html#module-key-differences"},{"type":"macro","title":"Shapex.schema/1","doc":"Macro to create a schema using a DSL.","ref":"Shapex.html#schema/1"},{"type":"function","title":"Shapex.validate/2","doc":"Validate data depending on the schema\n\n  Examples:\n\n    iex> some_schema = Shapex.Types.string()\n    iex> Shapex.validate(some_schema, \"string\")\n    :ok","ref":"Shapex.html#validate/2"},{"type":"module","title":"Shapex.Services.SchemaBuilder","doc":"This is implementation of the Schema DSL.\n  It accept expression of the DSL, parses it and generates shapex schema.","ref":"Shapex.Services.SchemaBuilder.html"},{"type":"function","title":"Shapex.Services.SchemaBuilder.call/1","doc":"","ref":"Shapex.Services.SchemaBuilder.html#call/1"},{"type":"behaviour","title":"Shapex.Type","doc":"A behaviour for defining custom types.\n\n  Example:\n\n  ```elixir\n  defmodule Email do\n    @behaviour Shapex.Type\n\n    @type t :: %__MODULE__{\n            allowed_domains: [String.t()]\n          }\n\n    defstruct [:allowed_domains]\n\n    def validate(schema, value) do\n      results = case String.split(value, \"@\") do\n        [_, domain] when domain in schema.allowed_domains -> :ok\n        _ -> {:error, %{type: \"Invalid email\"}}\n      end\n\n      if results == %{} do\n        :ok\n      else\n        {:error, results}\n      end\n    end\n  end\n\n  Shapex.validate(%Email{allowed_domains: [\"example.com\"]}, \"ilya@example.com\")\n\n  :ok\n  ```","ref":"Shapex.Type.html"},{"type":"callback","title":"Shapex.Type.validate/2","doc":"","ref":"Shapex.Type.html#c:validate/2"},{"type":"type","title":"Shapex.Type.t/0","doc":"","ref":"Shapex.Type.html#t:t/0"},{"type":"module","title":"Shapex.Types","doc":"Module that contains helper functions to define schema.","ref":"Shapex.Types.html"},{"type":"function","title":"Shapex.Types.any/0","doc":"Any type for Shapex.","ref":"Shapex.Types.html#any/0"},{"type":"function","title":"Example - Shapex.Types.any/0","doc":"any()","ref":"Shapex.Types.html#any/0-example"},{"type":"function","title":"Shapex.Types.atom/1","doc":"Atom type for Shapex.\n\n  Validations:\n    - :eq - equality check\n    - :neq - not equal check","ref":"Shapex.Types.html#atom/1"},{"type":"function","title":"Example - Shapex.Types.atom/1","doc":"atom(eq: :ok)","ref":"Shapex.Types.html#atom/1-example"},{"type":"function","title":"Shapex.Types.boolean/1","doc":"Boolean type for Shapex.","ref":"Shapex.Types.html#boolean/1"},{"type":"function","title":"Example - Shapex.Types.boolean/1","doc":"boolean(true)","ref":"Shapex.Types.html#boolean/1-example"},{"type":"function","title":"Shapex.Types.dict/2","doc":"Record type for Shapex. It's like a map, but fields are not pre-defined, useful for dictionaries.","ref":"Shapex.Types.html#dict/2"},{"type":"function","title":"Example - Shapex.Types.dict/2","doc":"schema = dict(string(), string())\n\n    value = %{\n      \"id1\" => \"value1\",\n      \"id2\" => \"value2\"\n      \"id3\" => \"value3\"\n    }\n\n    Shapex.validate(schema, value)\n\n    # :ok","ref":"Shapex.Types.html#dict/2-example"},{"type":"function","title":"Shapex.Types.enum/1","doc":"Enum type for Shapex.","ref":"Shapex.Types.html#enum/1"},{"type":"function","title":"Example - Shapex.Types.enum/1","doc":"enum([integer(), string()])","ref":"Shapex.Types.html#enum/1-example"},{"type":"function","title":"Shapex.Types.float/1","doc":"Float type for Shapex.\n  Validation rules:\n    - :gt - greater than\n    - :gte - greater than or equal\n    - :lt - less than\n    - :lte - less than or equal\n    - :eq - equal\n    - :neq - not equal\n    - :in - in list\n    - :not_in - not in list","ref":"Shapex.Types.html#float/1"},{"type":"function","title":"Shapex.Types.integer/1","doc":"Integer type for Shapex.\n  Validation rules:\n    - :gt - greater than\n    - :gte - greater than or equal\n    - :lt - less than\n    - :lte - less than or equal\n    - :eq - equal\n    - :neq - not equal\n    - :in - in list\n    - :not_in - not in list\n    - :custom - custom validation function","ref":"Shapex.Types.html#integer/1"},{"type":"function","title":"Example - Shapex.Types.integer/1","doc":"integer(\n      gt: 10,\n      gte: 10,\n      lt: 20,\n      lte: 20,\n      eq: 15,\n      neq: 10,\n      in: [1, 2, 3],\n      not_in: [4, 5, 6],\n      custom: fn value ->\n        if value > 10 do\n          \"Value must be less than 10\"\n        end\n      end\n    )","ref":"Shapex.Types.html#integer/1-example"},{"type":"function","title":"Shapex.Types.list/1","doc":"List type for Shapex.","ref":"Shapex.Types.html#list/1"},{"type":"function","title":"Example - Shapex.Types.list/1","doc":"list(integer())","ref":"Shapex.Types.html#list/1-example"},{"type":"function","title":"Shapex.Types.map/1","doc":"Map type for Shapex.\n  Fields required by default, if you want make them optional use &optional/1","ref":"Shapex.Types.html#map/1"},{"type":"function","title":"Example - Shapex.Types.map/1","doc":"map(%{\n      name: string(),\n      age: integer()\n    })","ref":"Shapex.Types.html#map/1-example"},{"type":"function","title":"Shapex.Types.number/1","doc":"Number type for Shapex.","ref":"Shapex.Types.html#number/1"},{"type":"function","title":"Shapex.Types.optional/1","doc":"Support function to make map field optional","ref":"Shapex.Types.html#optional/1"},{"type":"function","title":"Example - Shapex.Types.optional/1","doc":"map(%{\n      optional(:age) => integer()\n    })","ref":"Shapex.Types.html#optional/1-example"},{"type":"function","title":"Shapex.Types.string/1","doc":"String type for Shapex.\n\nValidation rules:\n  - :min_length - minimum length\n  - :max_length - maximum length\n  - :length - exact length\n  - :in - in list\n  - :not_in - not in list\n  - :custom - custom validation function","ref":"Shapex.Types.html#string/1"},{"type":"module","title":"Shapex.Types.Any","doc":"This module provides a type for validating any value.","ref":"Shapex.Types.Any.html"},{"type":"type","title":"Shapex.Types.Any.t/0","doc":"","ref":"Shapex.Types.Any.html#t:t/0"},{"type":"module","title":"Shapex.Types.Atom","doc":"This module provides a type for validating atoms.\n\nIt's better to use the Shapex.Types.atom/1 function to create a schema,\nsince it covers implementation details and provides a more user-friendly API.","ref":"Shapex.Types.Atom.html"},{"type":"function","title":"Shapex.Types.Atom.validate/2","doc":"","ref":"Shapex.Types.Atom.html#validate/2"},{"type":"type","title":"Shapex.Types.Atom.atom_or_tuple/0","doc":"","ref":"Shapex.Types.Atom.html#t:atom_or_tuple/0"},{"type":"type","title":"Shapex.Types.Atom.t/0","doc":"","ref":"Shapex.Types.Atom.html#t:t/0"},{"type":"type","title":"Shapex.Types.Atom.validation/0","doc":"","ref":"Shapex.Types.Atom.html#t:validation/0"},{"type":"module","title":"Shapex.Types.Boolean","doc":"This module provides a type for validating booleans.\n\nIt's better to use the Shapex.Types.boolean/1 function to create a schema,\nsince it covers implementation details and provides a more user-friendly API.","ref":"Shapex.Types.Boolean.html"},{"type":"type","title":"Shapex.Types.Boolean.boolean_or_tuple/0","doc":"","ref":"Shapex.Types.Boolean.html#t:boolean_or_tuple/0"},{"type":"type","title":"Shapex.Types.Boolean.t/0","doc":"","ref":"Shapex.Types.Boolean.html#t:t/0"},{"type":"module","title":"Shapex.Types.Dict","doc":"This module provides a type for validating dictionaries.\n\nRecords are a key-value pair where the key is a string and the value is any type.\n\nDifference between a dict and a map:\n- A dict is a key-value pair where you validate that the key has the type you expect, and the value has the type you expect.\n- A map is like a struct since you define set of keys and their values that you expect.\n\nIt's better to use the Shapex.Types.dict/2 function to create a schema,\nsince it covers implementation details and provides a more user-friendly API.","ref":"Shapex.Types.Dict.html"},{"type":"type","title":"Shapex.Types.Dict.t/0","doc":"","ref":"Shapex.Types.Dict.html#t:t/0"},{"type":"module","title":"Shapex.Types.Enum","doc":"This module provides a type for validating enums.\n\nEnums are a list of possible values that a value can be.\n\nIt's better to use the Shapex.Types.enum/1 function to create a schema,\nsince it covers implementation details and provides a more user-friendly API.","ref":"Shapex.Types.Enum.html"},{"type":"function","title":"Shapex.Types.Enum.validate/2","doc":"","ref":"Shapex.Types.Enum.html#validate/2"},{"type":"type","title":"Shapex.Types.Enum.t/0","doc":"","ref":"Shapex.Types.Enum.html#t:t/0"},{"type":"module","title":"Shapex.Types.Float","doc":"This module provides a type for validating floats.\n\nExisting validations:\n- :gt - greater than\n- :gte - greater than or equal to\n- :lt - less than\n- :lte - less than or equal to\n- :eq - equal\n- :neq - not equal\n- :in - checks if the value is in a list\n- :not_in - checks if the value is not in a list\n\nIt's better to use the Shapex.Types.float/1 function to create a schema,\nsince it covers implementation details and provides a more user-friendly API.","ref":"Shapex.Types.Float.html"},{"type":"function","title":"Shapex.Types.Float.validate/2","doc":"","ref":"Shapex.Types.Float.html#validate/2"},{"type":"type","title":"Shapex.Types.Float.float_list_or_tuple/0","doc":"","ref":"Shapex.Types.Float.html#t:float_list_or_tuple/0"},{"type":"type","title":"Shapex.Types.Float.float_or_tuple/0","doc":"","ref":"Shapex.Types.Float.html#t:float_or_tuple/0"},{"type":"type","title":"Shapex.Types.Float.t/0","doc":"","ref":"Shapex.Types.Float.html#t:t/0"},{"type":"type","title":"Shapex.Types.Float.validations/0","doc":"","ref":"Shapex.Types.Float.html#t:validations/0"},{"type":"module","title":"Shapex.Types.Integer","doc":"This module provides a type for validating integers.\n\n  Existing validations:\n  - :gt - greater than\n  - :gte - greater than or equal to\n  - :lt - less than\n  - :lte - less than or equal to\n  - :eq - equal\n  - :neq - not equal\n  - :in - checks if the value is in a list\n  - :not_in - checks if the value is not in a list\n\n  It's better to use the Shapex.Types.integer/1 function to create a schema,\n  since it covers implementation details and provides a more user-friendly API.","ref":"Shapex.Types.Integer.html"},{"type":"type","title":"Shapex.Types.Integer.t/0","doc":"","ref":"Shapex.Types.Integer.html#t:t/0"},{"type":"type","title":"Shapex.Types.Integer.target/1","doc":"","ref":"Shapex.Types.Integer.html#t:target/1"},{"type":"type","title":"Shapex.Types.Integer.validation_rules/0","doc":"","ref":"Shapex.Types.Integer.html#t:validation_rules/0"},{"type":"module","title":"Shapex.Types.List","doc":"Module that contains List type for Shapex.\n\nIt's better to use the Shapex.Types.list/1 function to create a schema,\nsince it covers implementation details and provides a more user-friendly API.","ref":"Shapex.Types.List.html"},{"type":"function","title":"Shapex.Types.List.validate/2","doc":"","ref":"Shapex.Types.List.html#validate/2"},{"type":"type","title":"Shapex.Types.List.t/0","doc":"","ref":"Shapex.Types.List.html#t:t/0"},{"type":"module","title":"Shapex.Types.Map","doc":"This module provides a type for validating maps.\n\nMaps have a schema that defines the keys and their respective types.\n\nIt's better to use the Shapex.Types.map/1 function to create a schema,\nsince it covers implementation details and provides a more user-friendly API.","ref":"Shapex.Types.Map.html"},{"type":"function","title":"Shapex.Types.Map.validate/2","doc":"","ref":"Shapex.Types.Map.html#validate/2"},{"type":"type","title":"Shapex.Types.Map.key/0","doc":"","ref":"Shapex.Types.Map.html#t:key/0"},{"type":"type","title":"Shapex.Types.Map.t/0","doc":"","ref":"Shapex.Types.Map.html#t:t/0"},{"type":"module","title":"Shapex.Types.Nil","doc":"This module provides a type for validating nil values.","ref":"Shapex.Types.Nil.html"},{"type":"function","title":"Shapex.Types.Nil.validate/2","doc":"","ref":"Shapex.Types.Nil.html#validate/2"},{"type":"module","title":"Shapex.Types.Number","doc":"This module provides a type for validating numbers.\n\n  Existing validations:\n  - :gt - greater than\n  - :gte - greater than or equal to\n  - :lt - less than\n  - :lte - less than or equal to\n  - :eq - equal\n  - :neq - not equal\n  - :in - checks if the value is in a list\n  - :not_in - checks if the value is not in a list\n\n  It's better to use the Shapex.Types.number/1 function to create a schema,\n  since it covers implementation details and provides a more user-friendly API.","ref":"Shapex.Types.Number.html"},{"type":"type","title":"Shapex.Types.Number.t/0","doc":"","ref":"Shapex.Types.Number.html#t:t/0"},{"type":"type","title":"Shapex.Types.Number.target/1","doc":"","ref":"Shapex.Types.Number.html#t:target/1"},{"type":"type","title":"Shapex.Types.Number.validation_rules/0","doc":"","ref":"Shapex.Types.Number.html#t:validation_rules/0"},{"type":"module","title":"Shapex.Types.String","doc":"This module provides a type for validating strings.\n\nIt's better to use the Shapex.Types.string/1 function to create a schema,\nsince it covers implementation details and provides a more user-friendly API.\n\nExisting validations:\n- :min_length - minimum length\n- :max_length - maximum length\n- :length - exact length\n- :eq - equal\n- :neq - not equal\n- :regex - regex match","ref":"Shapex.Types.String.html"},{"type":"function","title":"Shapex.Types.String.validate/2","doc":"","ref":"Shapex.Types.String.html#validate/2"},{"type":"type","title":"Shapex.Types.String.t/0","doc":"","ref":"Shapex.Types.String.html#t:t/0"},{"type":"type","title":"Shapex.Types.String.type_or_tuple/1","doc":"","ref":"Shapex.Types.String.html#t:type_or_tuple/1"},{"type":"type","title":"Shapex.Types.String.validation_rule/0","doc":"","ref":"Shapex.Types.String.html#t:validation_rule/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}